@@ -1,13 +1,13 @@
 /* Test file for mpfr_mul.
 
-Copyright 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
-Contributed by the Arenaire and Cacao projects, INRIA.
+Copyright 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
+Contributed by the Arenaire and Caramel projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
 The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
-the Free Software Foundation; either version 2.1 of the License, or (at your
+the Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 The GNU MPFR Library is distributed in the hope that it will be useful, but
@@ -16,9 +16,9 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
-MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
+http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -27,10 +27,10 @@ MA 02110-1301, USA. */
 
 #ifdef CHECK_EXTERNAL
 static int
-test_mul (mpfr_ptr a, mpfr_srcptr b, mpfr_srcptr c, mp_rnd_t rnd_mode)
+test_mul (mpfr_ptr a, mpfr_srcptr b, mpfr_srcptr c, mpfr_rnd_t rnd_mode)
 {
   int res;
-  int ok = rnd_mode == GMP_RNDN && mpfr_number_p (b) && mpfr_number_p (c);
+  int ok = rnd_mode == MPFR_RNDN && mpfr_number_p (b) && mpfr_number_p (c);
   if (ok)
     {
       mpfr_print_raw (b);
@@ -50,13 +50,10 @@ test_mul (mpfr_ptr a, mpfr_srcptr b, mpfr_srcptr c, mp_rnd_t rnd_mode)
 #define test_mul mpfr_mul
 #endif
 
-/* Workaround for sparc gcc 2.95.x bug, see notes in tadd.c. */
-#define check(x,y,rnd_mode,px,py,pz,res)  pcheck(x,y,res,rnd_mode,px,py,pz)
-
-/* checks that x*y gives the right result */
+/* checks that xs * ys gives the expected result res */
 static void
-pcheck (const char *xs, const char *ys, const char *res, mp_rnd_t rnd_mode,
-        unsigned int px, unsigned int py, unsigned int pz)
+check (const char *xs, const char *ys, mpfr_rnd_t rnd_mode,
+        unsigned int px, unsigned int py, unsigned int pz, const char *res)
 {
   mpfr_t xx, yy, zz;
 
@@ -71,7 +68,7 @@ pcheck (const char *xs, const char *ys, const char *res, mp_rnd_t rnd_mode,
       printf ("(1)mpfr_mul failed for x=%s y=%s with rnd=%s\n",
               xs, ys, mpfr_print_rnd_mode (rnd_mode));
       printf ("correct is %s, mpfr_mul gives ", res);
-      mpfr_out_str(stdout, 10, 0, zz, GMP_RNDN);
+      mpfr_out_str(stdout, 10, 0, zz, MPFR_RNDN);
       /*
         printf("\nBinary forms:\nxx=");
         mpfr_print_binary (xx);
@@ -89,7 +86,7 @@ pcheck (const char *xs, const char *ys, const char *res, mp_rnd_t rnd_mode,
 }
 
 static void
-check53 (const char *xs, const char *ys, mp_rnd_t rnd_mode, const char *zs)
+check53 (const char *xs, const char *ys, mpfr_rnd_t rnd_mode, const char *zs)
 {
   mpfr_t xx, yy, zz;
 
@@ -102,7 +99,7 @@ check53 (const char *xs, const char *ys, mp_rnd_t rnd_mode, const char *zs)
       printf ("(2) mpfr_mul failed for x=%s y=%s with rnd=%s\n",
               xs, ys, mpfr_print_rnd_mode(rnd_mode));
       printf ("correct result is %s,\n mpfr_mul gives ", zs);
-      mpfr_out_str(stdout, 10, 0, zz, GMP_RNDN);
+      mpfr_out_str(stdout, 10, 0, zz, MPFR_RNDN);
       /*
         printf("\nBinary forms:\nxx=");
         mpfr_print_binary (xx);
@@ -121,7 +118,7 @@ check53 (const char *xs, const char *ys, mp_rnd_t rnd_mode, const char *zs)
 
 /* checks that x*y gives the right result with 24 bits of precision */
 static void
-check24 (const char *xs, const char *ys, mp_rnd_t rnd_mode, const char *zs)
+check24 (const char *xs, const char *ys, mpfr_rnd_t rnd_mode, const char *zs)
 {
   mpfr_t xx, yy, zz;
 
@@ -134,7 +131,7 @@ check24 (const char *xs, const char *ys, mp_rnd_t rnd_mode, const char *zs)
       printf ("(3) mpfr_mul failed for x=%s y=%s with "
               "rnd=%s\n", xs, ys, mpfr_print_rnd_mode(rnd_mode));
       printf ("correct result is gives %s, mpfr_mul gives ", zs);
-      mpfr_out_str(stdout, 10, 0, zz, GMP_RNDN);
+      mpfr_out_str(stdout, 10, 0, zz, MPFR_RNDN);
       putchar('\n');
       exit (1);
     }
@@ -146,45 +143,45 @@ check24 (const char *xs, const char *ys, mp_rnd_t rnd_mode, const char *zs)
 static void
 check_float (void)
 {
-  check24("8388609.0",  "8388609.0", GMP_RNDN, "70368760954880.0");
-  check24("16777213.0", "8388609.0", GMP_RNDN, "140737479966720.0");
-  check24("8388611.0",  "8388609.0", GMP_RNDN, "70368777732096.0");
-  check24("12582911.0", "8388610.0", GMP_RNDN, "105553133043712.0");
-  check24("12582914.0", "8388610.0", GMP_RNDN, "105553158209536.0");
-  check24("13981013.0", "8388611.0", GMP_RNDN, "117281279442944.0");
-  check24("11184811.0", "8388611.0", GMP_RNDN, "93825028587520.0");
-  check24("11184810.0", "8388611.0", GMP_RNDN, "93825020198912.0");
-  check24("13981014.0", "8388611.0", GMP_RNDN, "117281287831552.0");
+  check24("8388609.0",  "8388609.0", MPFR_RNDN, "70368760954880.0");
+  check24("16777213.0", "8388609.0", MPFR_RNDN, "140737479966720.0");
+  check24("8388611.0",  "8388609.0", MPFR_RNDN, "70368777732096.0");
+  check24("12582911.0", "8388610.0", MPFR_RNDN, "105553133043712.0");
+  check24("12582914.0", "8388610.0", MPFR_RNDN, "105553158209536.0");
+  check24("13981013.0", "8388611.0", MPFR_RNDN, "117281279442944.0");
+  check24("11184811.0", "8388611.0", MPFR_RNDN, "93825028587520.0");
+  check24("11184810.0", "8388611.0", MPFR_RNDN, "93825020198912.0");
+  check24("13981014.0", "8388611.0", MPFR_RNDN, "117281287831552.0");
 
-  check24("8388609.0",  "8388609.0", GMP_RNDZ, "70368760954880.0");
-  check24("16777213.0", "8388609.0", GMP_RNDZ, "140737471578112.0");
-  check24("8388611.0",  "8388609.0", GMP_RNDZ, "70368777732096.0");
-  check24("12582911.0", "8388610.0", GMP_RNDZ, "105553124655104.0");
-  check24("12582914.0", "8388610.0", GMP_RNDZ, "105553158209536.0");
-  check24("13981013.0", "8388611.0", GMP_RNDZ, "117281271054336.0");
-  check24("11184811.0", "8388611.0", GMP_RNDZ, "93825028587520.0");
-  check24("11184810.0", "8388611.0", GMP_RNDZ, "93825011810304.0");
-  check24("13981014.0", "8388611.0", GMP_RNDZ, "117281287831552.0");
+  check24("8388609.0",  "8388609.0", MPFR_RNDZ, "70368760954880.0");
+  check24("16777213.0", "8388609.0", MPFR_RNDZ, "140737471578112.0");
+  check24("8388611.0",  "8388609.0", MPFR_RNDZ, "70368777732096.0");
+  check24("12582911.0", "8388610.0", MPFR_RNDZ, "105553124655104.0");
+  check24("12582914.0", "8388610.0", MPFR_RNDZ, "105553158209536.0");
+  check24("13981013.0", "8388611.0", MPFR_RNDZ, "117281271054336.0");
+  check24("11184811.0", "8388611.0", MPFR_RNDZ, "93825028587520.0");
+  check24("11184810.0", "8388611.0", MPFR_RNDZ, "93825011810304.0");
+  check24("13981014.0", "8388611.0", MPFR_RNDZ, "117281287831552.0");
 
-  check24("8388609.0",  "8388609.0", GMP_RNDU, "70368769343488.0");
-  check24("16777213.0", "8388609.0", GMP_RNDU, "140737479966720.0");
-  check24("8388611.0",  "8388609.0", GMP_RNDU, "70368786120704.0");
-  check24("12582911.0", "8388610.0", GMP_RNDU, "105553133043712.0");
-  check24("12582914.0", "8388610.0", GMP_RNDU, "105553166598144.0");
-  check24("13981013.0", "8388611.0", GMP_RNDU, "117281279442944.0");
-  check24("11184811.0", "8388611.0", GMP_RNDU, "93825036976128.0");
-  check24("11184810.0", "8388611.0", GMP_RNDU, "93825020198912.0");
-  check24("13981014.0", "8388611.0", GMP_RNDU, "117281296220160.0");
+  check24("8388609.0",  "8388609.0", MPFR_RNDU, "70368769343488.0");
+  check24("16777213.0", "8388609.0", MPFR_RNDU, "140737479966720.0");
+  check24("8388611.0",  "8388609.0", MPFR_RNDU, "70368786120704.0");
+  check24("12582911.0", "8388610.0", MPFR_RNDU, "105553133043712.0");
+  check24("12582914.0", "8388610.0", MPFR_RNDU, "105553166598144.0");
+  check24("13981013.0", "8388611.0", MPFR_RNDU, "117281279442944.0");
+  check24("11184811.0", "8388611.0", MPFR_RNDU, "93825036976128.0");
+  check24("11184810.0", "8388611.0", MPFR_RNDU, "93825020198912.0");
+  check24("13981014.0", "8388611.0", MPFR_RNDU, "117281296220160.0");
 
-  check24("8388609.0",  "8388609.0", GMP_RNDD, "70368760954880.0");
-  check24("16777213.0", "8388609.0", GMP_RNDD, "140737471578112.0");
-  check24("8388611.0",  "8388609.0", GMP_RNDD, "70368777732096.0");
-  check24("12582911.0", "8388610.0", GMP_RNDD, "105553124655104.0");
-  check24("12582914.0", "8388610.0", GMP_RNDD, "105553158209536.0");
-  check24("13981013.0", "8388611.0", GMP_RNDD, "117281271054336.0");
-  check24("11184811.0", "8388611.0", GMP_RNDD, "93825028587520.0");
-  check24("11184810.0", "8388611.0", GMP_RNDD, "93825011810304.0");
-  check24("13981014.0", "8388611.0", GMP_RNDD, "117281287831552.0");
+  check24("8388609.0",  "8388609.0", MPFR_RNDD, "70368760954880.0");
+  check24("16777213.0", "8388609.0", MPFR_RNDD, "140737471578112.0");
+  check24("8388611.0",  "8388609.0", MPFR_RNDD, "70368777732096.0");
+  check24("12582911.0", "8388610.0", MPFR_RNDD, "105553124655104.0");
+  check24("12582914.0", "8388610.0", MPFR_RNDD, "105553158209536.0");
+  check24("13981013.0", "8388611.0", MPFR_RNDD, "117281271054336.0");
+  check24("11184811.0", "8388611.0", MPFR_RNDD, "93825028587520.0");
+  check24("11184810.0", "8388611.0", MPFR_RNDD, "93825011810304.0");
+  check24("13981014.0", "8388611.0", MPFR_RNDD, "117281287831552.0");
 }
 
 /* check sign of result */
@@ -195,13 +192,13 @@ check_sign (void)
 
   mpfr_init2 (a, 53);
   mpfr_init2 (b, 53);
-  mpfr_set_si (a, -1, GMP_RNDN);
-  mpfr_set_ui (b, 2, GMP_RNDN);
-  test_mul(a, b, b, GMP_RNDN);
+  mpfr_set_si (a, -1, MPFR_RNDN);
+  mpfr_set_ui (b, 2, MPFR_RNDN);
+  test_mul(a, b, b, MPFR_RNDN);
   if (mpfr_cmp_ui (a, 4) )
     {
       printf ("2.0*2.0 gives \n");
-      mpfr_out_str(stdout, 10, 0, a, GMP_RNDN);
+      mpfr_out_str(stdout, 10, 0, a, MPFR_RNDN);
       putchar('\n');
       exit (1);
     }
@@ -213,9 +210,9 @@ static void
 check_exact (void)
 {
   mpfr_t a, b, c, d;
-  mp_prec_t prec;
+  mpfr_prec_t prec;
   int i, inexact;
-  mp_rnd_t rnd;
+  mpfr_rnd_t rnd;
 
   mpfr_init (a);
   mpfr_init (b);
@@ -227,7 +224,7 @@ check_exact (void)
   mpfr_set_prec (c, 32);
   mpfr_set_str_binary (a, "1.1000111011000100e-1");
   mpfr_set_str_binary (b, "1.0010001111100111e-1");
-  if (test_mul (c, a, b, GMP_RNDZ))
+  if (test_mul (c, a, b, MPFR_RNDZ))
     {
       printf ("wrong return value (1)\n");
       exit (1);
@@ -283,17 +280,17 @@ static void
 check_max(void)
 {
   mpfr_t xx, yy, zz;
-  mp_exp_t emin;
+  mpfr_exp_t emin;
 
   mpfr_init2(xx, 4);
   mpfr_init2(yy, 4);
   mpfr_init2(zz, 4);
   mpfr_set_str1 (xx, "0.68750");
-  mpfr_mul_2si(xx, xx, MPFR_EMAX_DEFAULT/2, GMP_RNDN);
+  mpfr_mul_2si(xx, xx, MPFR_EMAX_DEFAULT/2, MPFR_RNDN);
   mpfr_set_str1 (yy, "0.68750");
-  mpfr_mul_2si(yy, yy, MPFR_EMAX_DEFAULT - MPFR_EMAX_DEFAULT/2 + 1, GMP_RNDN);
+  mpfr_mul_2si(yy, yy, MPFR_EMAX_DEFAULT - MPFR_EMAX_DEFAULT/2 + 1, MPFR_RNDN);
   mpfr_clear_flags();
-  test_mul(zz, xx, yy, GMP_RNDU);
+  test_mul(zz, xx, yy, MPFR_RNDU);
   if (!(mpfr_overflow_p() && MPFR_IS_INF(zz)))
     {
       printf("check_max failed (should be an overflow)\n");
@@ -301,14 +298,14 @@ check_max(void)
     }
 
   mpfr_clear_flags();
-  test_mul(zz, xx, yy, GMP_RNDD);
+  test_mul(zz, xx, yy, MPFR_RNDD);
   if (mpfr_overflow_p() || MPFR_IS_INF(zz))
     {
       printf("check_max failed (should NOT be an overflow)\n");
       exit(1);
     }
   mpfr_set_str1 (xx, "0.93750");
-  mpfr_mul_2si(xx, xx, MPFR_EMAX_DEFAULT, GMP_RNDN);
+  mpfr_mul_2si(xx, xx, MPFR_EMAX_DEFAULT, MPFR_RNDN);
   if (!(MPFR_IS_FP(xx) && MPFR_IS_FP(zz)))
     {
       printf("check_max failed (internal error)\n");
@@ -317,9 +314,9 @@ check_max(void)
   if (mpfr_cmp(xx, zz) != 0)
     {
       printf("check_max failed: got ");
-      mpfr_out_str(stdout, 2, 0, zz, GMP_RNDZ);
+      mpfr_out_str(stdout, 2, 0, zz, MPFR_RNDZ);
       printf(" instead of ");
-      mpfr_out_str(stdout, 2, 0, xx, GMP_RNDZ);
+      mpfr_out_str(stdout, 2, 0, xx, MPFR_RNDZ);
       printf("\n");
       exit(1);
     }
@@ -329,7 +326,7 @@ check_max(void)
   set_emin (0);
   mpfr_set_str_binary (xx, "0.1E0");
   mpfr_set_str_binary (yy, "0.1E0");
-  test_mul (zz, xx, yy, GMP_RNDN);
+  test_mul (zz, xx, yy, MPFR_RNDN);
   /* exact result is 0.1E-1, which should round to 0 */
   MPFR_ASSERTN(mpfr_cmp_ui (zz, 0) == 0 && MPFR_IS_POS(zz));
   set_emin (emin);
@@ -341,7 +338,7 @@ check_max(void)
   mpfr_set_str_binary (xx, "0.1E0");
   mpfr_nextabove (xx);
   mpfr_set_str_binary (yy, "0.1E0");
-  test_mul (zz, xx, yy, GMP_RNDN);
+  test_mul (zz, xx, yy, MPFR_RNDN);
   /* exact result is just above 0.1E-1, which should round to minfloat */
   MPFR_ASSERTN(mpfr_cmp (zz, yy) == 0);
   set_emin (emin);
@@ -360,21 +357,21 @@ check_min(void)
   mpfr_init2(yy, 4);
   mpfr_init2(zz, 3);
   mpfr_set_str1(xx, "0.9375");
-  mpfr_mul_2si(xx, xx, MPFR_EMIN_DEFAULT/2, GMP_RNDN);
+  mpfr_mul_2si(xx, xx, MPFR_EMIN_DEFAULT/2, MPFR_RNDN);
   mpfr_set_str1(yy, "0.9375");
-  mpfr_mul_2si(yy, yy, MPFR_EMIN_DEFAULT - MPFR_EMIN_DEFAULT/2 - 1, GMP_RNDN);
-  test_mul(zz, xx, yy, GMP_RNDD);
+  mpfr_mul_2si(yy, yy, MPFR_EMIN_DEFAULT - MPFR_EMIN_DEFAULT/2 - 1, MPFR_RNDN);
+  test_mul(zz, xx, yy, MPFR_RNDD);
   if (mpfr_sgn(zz) != 0)
     {
       printf("check_min failed: got ");
-      mpfr_out_str(stdout, 2, 0, zz, GMP_RNDZ);
+      mpfr_out_str(stdout, 2, 0, zz, MPFR_RNDZ);
       printf(" instead of 0\n");
       exit(1);
     }
 
-  test_mul(zz, xx, yy, GMP_RNDU);
+  test_mul(zz, xx, yy, MPFR_RNDU);
   mpfr_set_str1 (xx, "0.5");
-  mpfr_mul_2si(xx, xx, MPFR_EMIN_DEFAULT, GMP_RNDN);
+  mpfr_mul_2si(xx, xx, MPFR_EMIN_DEFAULT, MPFR_RNDN);
   if (mpfr_sgn(xx) <= 0)
     {
       printf("check_min failed (internal error)\n");
@@ -383,9 +380,9 @@ check_min(void)
   if (mpfr_cmp(xx, zz) != 0)
     {
       printf("check_min failed: got ");
-      mpfr_out_str(stdout, 2, 0, zz, GMP_RNDZ);
+      mpfr_out_str(stdout, 2, 0, zz, MPFR_RNDZ);
       printf(" instead of ");
-      mpfr_out_str(stdout, 2, 0, xx, GMP_RNDZ);
+      mpfr_out_str(stdout, 2, 0, xx, MPFR_RNDZ);
       printf("\n");
       exit(1);
     }
@@ -406,33 +403,33 @@ check_nans (void)
 
   /* nan * 0 == nan */
   mpfr_set_nan (x);
-  mpfr_set_ui (y, 0L, GMP_RNDN);
-  test_mul (p, x, y, GMP_RNDN);
+  mpfr_set_ui (y, 0L, MPFR_RNDN);
+  test_mul (p, x, y, MPFR_RNDN);
   MPFR_ASSERTN (mpfr_nan_p (p));
 
   /* 1 * nan == nan */
-  mpfr_set_ui (x, 1L, GMP_RNDN);
+  mpfr_set_ui (x, 1L, MPFR_RNDN);
   mpfr_set_nan (y);
-  test_mul (p, x, y, GMP_RNDN);
+  test_mul (p, x, y, MPFR_RNDN);
   MPFR_ASSERTN (mpfr_nan_p (p));
 
   /* 0 * +inf == nan */
-  mpfr_set_ui (x, 0L, GMP_RNDN);
+  mpfr_set_ui (x, 0L, MPFR_RNDN);
   mpfr_set_nan (y);
-  test_mul (p, x, y, GMP_RNDN);
+  test_mul (p, x, y, MPFR_RNDN);
   MPFR_ASSERTN (mpfr_nan_p (p));
 
   /* +1 * +inf == +inf */
-  mpfr_set_ui (x, 1L, GMP_RNDN);
+  mpfr_set_ui (x, 1L, MPFR_RNDN);
   mpfr_set_inf (y, 1);
-  test_mul (p, x, y, GMP_RNDN);
+  test_mul (p, x, y, MPFR_RNDN);
   MPFR_ASSERTN (mpfr_inf_p (p));
   MPFR_ASSERTN (mpfr_sgn (p) > 0);
 
   /* -1 * +inf == -inf */
-  mpfr_set_si (x, -1L, GMP_RNDN);
+  mpfr_set_si (x, -1L, MPFR_RNDN);
   mpfr_set_inf (y, 1);
-  test_mul (p, x, y, GMP_RNDN);
+  test_mul (p, x, y, MPFR_RNDN);
   MPFR_ASSERTN (mpfr_inf_p (p));
   MPFR_ASSERTN (mpfr_sgn (p) < 0);
 
@@ -483,15 +480,15 @@ check_regression (void)
       exit (1);
     }
   get_string (s, fp);
-  mpfr_set_str (y, s, 16, GMP_RNDN);
+  mpfr_set_str (y, s, 16, MPFR_RNDN);
   get_string (s, fp);
-  mpfr_set_str (z, s, 16, GMP_RNDN);
-  i = mpfr_mul (x, y, z, GMP_RNDN);
+  mpfr_set_str (z, s, 16, MPFR_RNDN);
+  i = mpfr_mul (x, y, z, MPFR_RNDN);
   get_string (s, fp);
-  if (mpfr_cmp_str (x, s, 16, GMP_RNDN) != 0 || i != -1)
+  if (mpfr_cmp_str (x, s, 16, MPFR_RNDN) != 0 || i != -1)
     {
       printf ("Regression test 1 failed (i=%d, expected -1)\nx=", i);
-      mpfr_out_str (stdout, 16, 0, x, GMP_RNDN); putchar ('\n');
+      mpfr_out_str (stdout, 16, 0, x, MPFR_RNDN); putchar ('\n');
       exit (1);
     }
   fclose (fp);
@@ -500,14 +497,14 @@ check_regression (void)
   mpfr_set_prec (y, 606);
   mpfr_set_prec (z, 606);
 
-  mpfr_set_str (y, "-f.ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff92daefc3f8052ca9f58736564d9e93e62d324@-1", 16, GMP_RNDN);
-  mpfr_set_str (z, "-f.ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff92daefc3f8052ca9f58736564d9e93e62d324@-1", 16, GMP_RNDN);
-  i = mpfr_mul (x, y, z, GMP_RNDU);
-  mpfr_set_str (y, "f.ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff25b5df87f00a5953eb0e6cac9b3d27cc5a64c@-1", 16, GMP_RNDN);
+  mpfr_set_str (y, "-f.ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff92daefc3f8052ca9f58736564d9e93e62d324@-1", 16, MPFR_RNDN);
+  mpfr_set_str (z, "-f.ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff92daefc3f8052ca9f58736564d9e93e62d324@-1", 16, MPFR_RNDN);
+  i = mpfr_mul (x, y, z, MPFR_RNDU);
+  mpfr_set_str (y, "f.ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff25b5df87f00a5953eb0e6cac9b3d27cc5a64c@-1", 16, MPFR_RNDN);
   if (mpfr_cmp (x, y) || i <= 0)
     {
       printf ("Regression test (2) failed! (i=%d - Expected 1)\n", i);
-      mpfr_out_str (stdout, 16, 0, x, GMP_RNDN); putchar ('\n');
+      mpfr_out_str (stdout, 16, 0, x, MPFR_RNDN); putchar ('\n');
       exit (1);
     }
 
@@ -515,18 +512,18 @@ check_regression (void)
   mpfr_set_prec (y, 92);
   mpfr_set_prec (z, 1023);
 
-  mpfr_set_str (y, "6.9b8c8498882770d8038c3b0@-1", 16, GMP_RNDN);
-  mpfr_set_str (z, "7.44e24b986e7fb296f1e936ce749fec3504cbf0d5ba769466b1c9f1578115efd5d29b4c79271191a920a99280c714d3a657ad6e3afbab77ffce9d697e9bb9110e26d676069afcea8b69f1d1541f2365042d80a97c21dcccd8ace4f1bb58b49922003e738e6f37bb82ef653cb2e87f763974e6ae50ae54e7724c38b80653e3289@255", 16, GMP_RNDN);
-  i = mpfr_mul (x, y, z, GMP_RNDU);
+  mpfr_set_str (y, "6.9b8c8498882770d8038c3b0@-1", 16, MPFR_RNDN);
+  mpfr_set_str (z, "7.44e24b986e7fb296f1e936ce749fec3504cbf0d5ba769466b1c9f1578115efd5d29b4c79271191a920a99280c714d3a657ad6e3afbab77ffce9d697e9bb9110e26d676069afcea8b69f1d1541f2365042d80a97c21dcccd8ace4f1bb58b49922003e738e6f37bb82ef653cb2e87f763974e6ae50ae54e7724c38b80653e3289@255", 16, MPFR_RNDN);
+  i = mpfr_mul (x, y, z, MPFR_RNDU);
   mpfr_set_prec (y, 184);
   mpfr_set_str (y, "3.0080038f2ac5054e3e71ccbb95f76aaab2221715025a28@255",
-                16, GMP_RNDN);
+                16, MPFR_RNDN);
   if (mpfr_cmp (x, y) || i <= 0)
     {
       printf ("Regression test (4) failed! (i=%d - expected 1)\n", i);
       printf ("Ref: 3.0080038f2ac5054e3e71ccbb95f76aaab2221715025a28@255\n"
               "Got: ");
-      mpfr_out_str (stdout, 16, 0, x, GMP_RNDN);
+      mpfr_out_str (stdout, 16, 0, x, MPFR_RNDN);
       printf ("\n");
       exit (1);
     }
@@ -537,20 +534,20 @@ check_regression (void)
   mpfr_set_str (y, "-f.fffffffffffffffffffffffffffffffffffffffffffffffffffffff"
 "fffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff"
 "ffffffffffffffffffffffffffffffffffffffffffffffffffffff99be91f83ec6f0ed28a3d42"
-"e6e9a327230345ea6@-1", 16, GMP_RNDN);
+"e6e9a327230345ea6@-1", 16, MPFR_RNDN);
   mpfr_set_str (z, "-f.fffffffffffffffffffffffffffffffffffffffffffffffffffffff"
 "fffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff"
 "ffffffffffffffffffffffffffffffffffffffffffffffffffffff99be91f83ec6f0ed28a3d42"
-                "e6e9a327230345ea6@-1", 16, GMP_RNDN);
-  i = mpfr_mul (x, y, z, GMP_RNDU);
+                "e6e9a327230345ea6@-1", 16, MPFR_RNDN);
+  i = mpfr_mul (x, y, z, MPFR_RNDU);
   mpfr_set_str (y, "f.ffffffffffffffffffffffffffffffffffffffffffffffffffffffff"
 "fffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff"
 "fffffffffffffffffffffffffffffffffffffffffffffffffffff337d23f07d8de1da5147a85c"
-"dd3464e46068bd4d@-1", 16, GMP_RNDN);
+"dd3464e46068bd4d@-1", 16, MPFR_RNDN);
   if (mpfr_cmp (x, y) || i <= 0)
     {
       printf ("Regression test (5) failed! (i=%d - expected 1)\n", i);
-      mpfr_out_str (stdout, 16, 0, x, GMP_RNDN);
+      mpfr_out_str (stdout, 16, 0, x, MPFR_RNDN);
       printf ("\n");
       exit (1);
     }
@@ -559,18 +556,18 @@ check_regression (void)
   mpfr_set_prec (x, 50);
   mpfr_set_prec (y, 40);
   mpfr_set_prec (z, 53);
-  mpfr_set_str (y, "4.1ffffffff8", 16, GMP_RNDN);
-  mpfr_set_str (z, "4.2000000ffe0000@-4", 16, GMP_RNDN);
-  i = mpfr_mul (x, y, z, GMP_RNDN);
-  if (mpfr_cmp_str (x, "1.104000041d6c0@-3", 16, GMP_RNDN) != 0
+  mpfr_set_str (y, "4.1ffffffff8", 16, MPFR_RNDN);
+  mpfr_set_str (z, "4.2000000ffe0000@-4", 16, MPFR_RNDN);
+  i = mpfr_mul (x, y, z, MPFR_RNDN);
+  if (mpfr_cmp_str (x, "1.104000041d6c0@-3", 16, MPFR_RNDN) != 0
       || i <= 0)
     {
       printf ("Regression test (6) failed! (i=%d - expected 1)\nx=", i);
-      mpfr_out_str (stdout, 16, 0, x, GMP_RNDN);
+      mpfr_out_str (stdout, 16, 0, x, MPFR_RNDN);
       printf ("\nMore prec=");
       mpfr_set_prec (x, 93);
-      mpfr_mul (x, y, z, GMP_RNDN);
-      mpfr_out_str (stdout, 16, 0, x, GMP_RNDN);
+      mpfr_mul (x, y, z, MPFR_RNDN);
+      mpfr_out_str (stdout, 16, 0, x, MPFR_RNDN);
       printf ("\n");
       exit (1);
     }
@@ -579,15 +576,15 @@ check_regression (void)
   mpfr_set_prec (y, 393);
   mpfr_set_str (y, "-1.921fb54442d18469898cc51701b839a252049c1114cf98e804177d"
                 "4c76273644a29410f31c6809bbdf2a33679a748636600",
-                16, GMP_RNDN);
-  i = mpfr_mul (x, y, y, GMP_RNDU);
+                16, MPFR_RNDN);
+  i = mpfr_mul (x, y, y, MPFR_RNDU);
   if (mpfr_cmp_str (x, "2.77a79937c8bbcb495b89b36602306b1c2159a8ff834288a19a08"
     "84094f1cda3dc426da61174c4544a173de83c2500f8bfea2e0569e3698",
-                    16, GMP_RNDN) != 0
+                    16, MPFR_RNDN) != 0
       || i <= 0)
     {
       printf ("Regression test (7) failed! (i=%d - expected 1)\nx=", i);
-      mpfr_out_str (stdout, 16, 0, x, GMP_RNDN);
+      mpfr_out_str (stdout, 16, 0, x, MPFR_RNDN);
       printf ("\n");
       exit (1);
     }
@@ -595,13 +592,13 @@ check_regression (void)
   mpfr_set_prec (x, 1023);
   mpfr_set_prec (y, 1023);
   mpfr_set_prec (z, 511);
-  mpfr_set_ui (x, 17, GMP_RNDN);
-  mpfr_set_ui (y, 42, GMP_RNDN);
-  i = mpfr_mul (z, x, y, GMP_RNDN);
+  mpfr_set_ui (x, 17, MPFR_RNDN);
+  mpfr_set_ui (y, 42, MPFR_RNDN);
+  i = mpfr_mul (z, x, y, MPFR_RNDN);
   if (mpfr_cmp_ui (z, 17*42) != 0 || i != 0)
     {
       printf ("Regression test (8) failed! (i=%d - expected 0)\nz=", i);
-      mpfr_out_str (stdout, 16, 0, z, GMP_RNDN);
+      mpfr_out_str (stdout, 16, 0, z, MPFR_RNDN);
       printf ("\n");
       exit (1);
     }
@@ -611,12 +608,12 @@ check_regression (void)
 
 #define TEST_FUNCTION test_mul
 #define TWO_ARGS
-#define RAND_FUNCTION(x) mpfr_random2(x, MPFR_LIMB_SIZE (x), randlimb () % 100)
+#define RAND_FUNCTION(x) mpfr_random2(x, MPFR_LIMB_SIZE (x), randlimb () % 100, RANDS)
 #include "tgeneric.c"
 
 /* multiplies x by 53-bit approximation of Pi */
 static int
-mpfr_mulpi (mpfr_ptr y, mpfr_srcptr x, mp_rnd_t r)
+mpfr_mulpi (mpfr_ptr y, mpfr_srcptr x, mpfr_rnd_t r)
 {
   mpfr_t z;
   int inex;
@@ -628,6 +625,25 @@ mpfr_mulpi (mpfr_ptr y, mpfr_srcptr x, mp_rnd_t r)
   return inex;
 }
 
+static void
+valgrind20110503 (void)
+{
+  mpfr_t a, b, c;
+
+  mpfr_init2 (a, 2);
+  mpfr_init2 (b, 2005);
+  mpfr_init2 (c, 2);
+
+  mpfr_set_ui (b, 5, MPFR_RNDN);
+  mpfr_nextabove (b);
+  mpfr_set_ui (c, 1, MPFR_RNDN);
+  mpfr_mul (a, b, c, MPFR_RNDZ);
+  /* After the call to mpfr_mulhigh_n, valgrind complains:
+     Conditional jump or move depends on uninitialised value(s) */
+
+  mpfr_clears (a, b, c, (mpfr_ptr) 0);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -637,30 +653,34 @@ main (int argc, char *argv[])
   check_exact ();
   check_float ();
 
-  check53("6.9314718055994530941514e-1", "0.0", GMP_RNDZ, "0.0");
-  check53("0.0", "6.9314718055994530941514e-1", GMP_RNDZ, "0.0");
+  check53("6.9314718055994530941514e-1", "0.0", MPFR_RNDZ, "0.0");
+  check53("0.0", "6.9314718055994530941514e-1", MPFR_RNDZ, "0.0");
   check_sign();
-  check53("-4.165000000e4", "-0.00004801920768307322868063274915", GMP_RNDN,
+  check53("-4.165000000e4", "-0.00004801920768307322868063274915", MPFR_RNDN,
           "2.0");
   check53("2.71331408349172961467e-08", "-6.72658901114033715233e-165",
-          GMP_RNDZ, "-1.8251348697787782844e-172");
-  check53("0.31869277231188065", "0.88642843322303122", GMP_RNDZ,
+          MPFR_RNDZ, "-1.8251348697787782844e-172");
+  check53("2.71331408349172961467e-08", "-6.72658901114033715233e-165",
+          MPFR_RNDA, "-1.8251348697787786e-172");
+  check53("0.31869277231188065", "0.88642843322303122", MPFR_RNDZ,
           "2.8249833483992453642e-1");
-  check("8.47622108205396074254e-01", "3.24039313247872939883e-01", GMP_RNDU,
+  check("8.47622108205396074254e-01", "3.24039313247872939883e-01", MPFR_RNDU,
         28, 45, 2, "0.375");
-  check("2.63978122803639081440e-01", "6.8378615379333496093e-1", GMP_RNDN,
+  check("8.47622108205396074254e-01", "3.24039313247872939883e-01", MPFR_RNDA,
+        28, 45, 2, "0.375");
+  check("2.63978122803639081440e-01", "6.8378615379333496093e-1", MPFR_RNDN,
         34, 23, 31, "0.180504585267044603");
-  check("1.0", "0.11835170935876249132", GMP_RNDU, 6, 41, 36,
+  check("1.0", "0.11835170935876249132", MPFR_RNDU, 6, 41, 36,
         "0.1183517093595583");
-  check53("67108865.0", "134217729.0", GMP_RNDN, "9.007199456067584e15");
-  check("1.37399642157394197284e-01", "2.28877275604219221350e-01", GMP_RNDN,
+  check53("67108865.0", "134217729.0", MPFR_RNDN, "9.007199456067584e15");
+  check("1.37399642157394197284e-01", "2.28877275604219221350e-01", MPFR_RNDN,
         49, 15, 32, "0.0314472340833162888");
   check("4.03160720978664954828e-01", "5.854828e-1"
-        /*"5.85483042917246621073e-01"*/, GMP_RNDZ,
+        /*"5.85483042917246621073e-01"*/, MPFR_RNDZ,
         51, 22, 32, "0.2360436821472831");
-  check("3.90798504668055102229e-14", "9.85394674650308388664e-04", GMP_RNDN,
+  check("3.90798504668055102229e-14", "9.85394674650308388664e-04", MPFR_RNDN,
         46, 22, 12, "0.385027296503914762e-16");
-  check("4.58687081072827851358e-01", "2.20543551472118792844e-01", GMP_RNDN,
+  check("4.58687081072827851358e-01", "2.20543551472118792844e-01", MPFR_RNDN,
         49, 3, 2, "0.09375");
   check_max();
   check_min();
@@ -670,6 +690,8 @@ main (int argc, char *argv[])
 
   data_check ("data/mulpi", mpfr_mulpi, "mpfr_mulpi");
 
+  valgrind20110503 ();
+
   tests_end_mpfr ();
   return 0;
 }
